package net.sjava.cassandra.manager;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import net.sjava.cassandra.model.Post;
import net.sjava.cassandra.util.ColumnUtil;
import net.sjava.cassandra.util.ConnectionUtil;
import net.sjava.cassandra.util.ConverterUtil;

import org.apache.cassandra.thrift.Cassandra;
import org.apache.cassandra.thrift.Column;
import org.apache.cassandra.thrift.ColumnOrSuperColumn;
import org.apache.cassandra.thrift.ColumnParent;
import org.apache.cassandra.thrift.ColumnPath;
import org.apache.cassandra.thrift.ConsistencyLevel;
import org.apache.cassandra.thrift.IndexClause;
import org.apache.cassandra.thrift.IndexExpression;
import org.apache.cassandra.thrift.IndexOperator;
import org.apache.cassandra.thrift.KeyRange;
import org.apache.cassandra.thrift.KeySlice;
import org.apache.cassandra.thrift.SlicePredicate;
//import org.apache.log4j.Logger;


/**
 * α ÿ  ó ϴ Ŵ Ŭ 
 * 
 * @author mcsong@gmail.com
 *
 */
public class PostManager {
	
	//private static final Logger LOG = Logger.getLogger(PostManager.class);
	private static String UTF8 ="UTF8";
	
	private Cassandra.Client client;
	private String cfName = "posts";
	private ColumnParent cp = new ColumnParent(cfName);
	
	// õ Ʈ ü Ű Ѵ. 
	private List<Long> keys = new ArrayList<Long>();
	
	// ̱ ü 
	public static PostManager instance = new PostManager();
	
	private PostManager() { 
		try {
			client  = ConnectionUtil.createConnection("BLOG");
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	public static PostManager instance() {
		return instance;
	}
	
	/**
	 * Ʈ ü  ȯѴ. 
	 * 
	 * @return
	 * @throws Exception
	 */
	public int getToalPostCount() throws Exception {
		return keys.size();
	}
	
	/**
	 * SlicePredicate ü ȯѴ. 
	 * 
	 * @return
	 * @throws IOException
	 */
	private SlicePredicate createSlicePredicate() throws IOException {
		SlicePredicate predicate = new SlicePredicate();
		predicate.addToColumn_names(ByteBuffer.wrap("id".getBytes("UTF8")));
		predicate.addToColumn_names(ByteBuffer.wrap("title".getBytes("UTF8")));
		predicate.addToColumn_names(ByteBuffer.wrap("post".getBytes("UTF8")));
		predicate.addToColumn_names(ByteBuffer.wrap("category_id".getBytes("UTF8")));
		predicate.addToColumn_names(ByteBuffer.wrap("post_date".getBytes("UTF8")));
		return predicate;
	}

	/**
	 * KeyRange ü ȯѴ. 
	 * @param key
	 * @return
	 * @throws IOException
	 */
	/*
	private KeyRange createKeyRange(long key) throws IOException {
		KeyRange keyRange = new KeyRange();
		keyRange.start_key = ConverterUtil.longToBytebuffer(key);
		keyRange.end_key = ConverterUtil.longToBytebuffer(key);
		keyRange.setCount(1);
		
		return keyRange;
	}
	*/
	
	/**
	 * Ű Ʈ ü ¡ ͸ ´.
	 * 
	 * @return
	 * @throws Exception
	 */
	public List<Post> getPosts(int count, int index) throws Exception {
		 
		int end = count * index;
		int start = end - count;
		
		if(start > getToalPostCount())
			return null;
		
		List<Post> posts = new ArrayList<Post>();
		
		for(int i = start; i < end ; i++) {
			if(i >= getToalPostCount())
				break;
			
			// Ű شϴ  ´.
			SlicePredicate predicate = this.createSlicePredicate();			

	        List<ColumnOrSuperColumn> columns = 
	        	client.get_slice(ConverterUtil.longToBytebuffer(this.keys.get(i)), cp, predicate, ConsistencyLevel.ONE);

	        Post post = Post.create();
	        post.setKey(this.keys.get(i));
	        String name = null;
	        for (ColumnOrSuperColumn cosc : columns) {
	        	name = ConverterUtil.byteBufferToString(cosc.column.name);
	            if("id".equals(name))
	            	post.setId(Long.parseLong( ConverterUtil.byteBufferToString(cosc.column.value)) );
	            
	            if("title".equals(name))
	            	post.setTitle(ConverterUtil.byteBufferToString(cosc.column.value) );
	            
	            if("post".equals(name))
	            	post.setPost(ConverterUtil.byteBufferToString(cosc.column.value) );
	            
	            if("category_id".equals(name))
	            	post.setCategoryId(ConverterUtil.byteBufferToString(cosc.column.value) );
	            
	            if("post_date".equals(name))
	            	post.setDate(cosc.column.value.getLong() );
	        }
	        
	        posts.add(post);
		}
		
		return posts;
	}
	
	
	/**
	 * Ʈ ߰Ѵ.
	 * 
	 * @param key
	 * @param title
	 * @param post
	 * @param categoryID
	 * @throws Exception
	 */
	public void addPost(long key, String title, String post, String categoryID) throws Exception {
		//LOG.debug(String.format("addPost(%s, %s, %s) start", title, post, categoryID));
		
		long timestamp = System.currentTimeMillis();
		
		Column idColumn = ColumnUtil.createColumn("id".getBytes(UTF8), String.valueOf(timestamp).getBytes(UTF8), timestamp);
    	Column titleColumn = ColumnUtil.createColumn("title".getBytes(UTF8), title.getBytes(UTF8), timestamp);
    	Column postColumn = ColumnUtil.createColumn("post".getBytes(UTF8), post.getBytes(UTF8), timestamp);
    	Column categoryIDColumn = ColumnUtil.createColumn("category_id".getBytes(UTF8), categoryID.getBytes(UTF8), timestamp);
    	Column dateColumn = ColumnUtil.createColumn("post_date".getBytes(UTF8), timestamp, timestamp);
    	
    	ByteBuffer rowKey = ConverterUtil.longToBytebuffer(key);
    	
    	client.insert(rowKey, cp, idColumn, ConsistencyLevel.ONE);
    	client.insert(rowKey, cp, titleColumn, ConsistencyLevel.ONE);
    	client.insert(rowKey, cp, postColumn, ConsistencyLevel.ONE);
    	client.insert(rowKey, cp, categoryIDColumn, ConsistencyLevel.ONE);
    	client.insert(rowKey, cp, dateColumn, ConsistencyLevel.ONE);
    	
		//LOG.debug(String.format("addPost(%s, %s, %s) end", title, post, categoryID));
	}
	
	/**
	 * Ʈ Ѵ. 
	 * 
	 * @param key
	 * @throws Exception
	 */
	public void deletePost(long key) throws Exception {
		//LOG.debug(String.format("deletePost(%d) start", key));
				
		long timestamp = System.currentTimeMillis();
		
		ColumnPath cPath = new ColumnPath();
		cPath.setColumn_family(cfName);
		
		client.remove(ConverterUtil.longToBytebuffer(key) , cPath, timestamp, ConsistencyLevel.ONE);
		//LOG.debug(String.format("deletePost(%d) end", key));
	}
	
	/**
	 * б ̸ Ű ε д. 
	 * 
	 * @throws Exception
	 */
	public void prepare() throws Exception {	
		SlicePredicate predicate = new SlicePredicate();
		predicate.addToColumn_names(ByteBuffer.wrap("id".getBytes("UTF8")));
		
		KeyRange keyRange = new KeyRange();
		keyRange.start_key = ByteBuffer.allocate(0);
		keyRange.end_key = ByteBuffer.allocate(0);
		keyRange.count = 10000;
				
		// Ű ̽ ȯȴ
		List<KeySlice> results = client.get_range_slices(cp, predicate, keyRange, ConsistencyLevel.ONE);
		
		// Ű  ֽ Ű   ´. 
		Collections.reverse(results);
		
		//  Ű Ű Ʈ Ѵ. 
		keys.clear();
		for (KeySlice keySlice : results) {
			keys.add(ByteBuffer.wrap(keySlice.getKey()).getLong());
		}
	}
	
	/**
	 * īװ б ̸ Ű ε д. 
	 * 
	 * @param categoryId
	 * @throws Exception
	 */
	public void prepareWithCategory(String categoryId) throws Exception {
		//LOG.debug(String.format("prepareWithCategory(%s) start", categoryId));
		
		// 
		IndexClause indexClause = new IndexClause();
		indexClause.start_key = ByteBuffer.allocate(0);
		IndexExpression indexExpression = new IndexExpression();
		indexExpression.column_name = ByteBuffer.wrap("category_id".getBytes("UTF-8"));
		indexExpression.value = ByteBuffer.wrap(categoryId.getBytes("UTF-8"));
		indexExpression.op = IndexOperator.EQ;
		
		indexClause.addToExpressions(indexExpression);
		
		SlicePredicate predicate = new SlicePredicate();
		predicate.addToColumn_names(ByteBuffer.wrap("category_id".getBytes("UTF8")));
				
		// Ű ̽ ȯȴ
		List<KeySlice> results = client.get_indexed_slices(cp, indexClause, predicate, ConsistencyLevel.ONE);
		
		// Ű  ֽ Ű   ´. 
		Collections.reverse(results);
	
		//  Ű Ű Ʈ Ѵ. 
		keys.clear();
		for (KeySlice keySlice : results)
			keys.add(ByteBuffer.wrap(keySlice.getKey()).getLong());
						
		//LOG.debug(String.format("prepareWithCategory(%s) end", categoryId));
	}
	
}
